"""
日志相关文件操作类
"""
import os
from sys import _getframe as get_run_info
import platform
import datetime

try:
    import xml.etree.cElementTree as XMLTree
except ImportError:
    import xml.etree.ElementTree as XMLTree


def create_log_file(file_name):
    """
    创建日志文件
    :param file_name: 文件名称
    :return: None
    """
    # 判断日志路径是否存在
    if not os.path.exists(os.path.dirname(file_name)):
        try:
            os.makedirs(os.path.dirname(file_name))
        except FileExistsError as e:
            print(str(e))

    # 判断日志文件是否存在
    if not os.path.exists(file_name):
        try:
            os.mknod(file_name)
        except AttributeError as e:
            print(str(e))
            f = open(file_name, 'w')
            f.close()


class FileOption:

    def __init__(self, conf_path, conf_deep):
        """
        类初始化
        :param conf_path: 配置文件路径
        :param conf_deep: 配置文件搜索深度
        """
        # 设置配置文件路径
        if conf_path == '':
            self.conf_path = os.getcwd() + '/eqlog.xml'
        elif conf_path[-4:] == '.xml':
            self.conf_path = conf_path
        elif conf_path[-1:] == '/' or conf_path[-1:] == '\\':
            self.conf_path = conf_path + 'eqlog.xml'
        else:
            self.conf_path = conf_path + '/eqlog.xml'
        # 配置文件读取深度
        self.conf_deep = conf_deep
        # 日志文件记录目录
        self.logfile_path = ''
        # 当前系统运行环境
        self.env = 'win_path' if platform.system().lower() == 'windows' else 'linux_path'
        # 控制台输出标志
        self.console = False

    def config_path_read(self, deep=1):
        """
        配置文件存在性校验
        :param deep: 搜索深度
        :return: None
        """
        if not os.path.exists(self.conf_path):
            eqlog_dir = os.path.dirname(self.conf_path) + '/eqconfig/eqlog.xml'
            if not os.path.exists(eqlog_dir) and deep <= self.conf_deep:
                self.conf_path = os.path.abspath(os.path.join(os.path.dirname(self.conf_path), "..")) + '/eqlog.xml'
                self.config_path_read(deep + 1)
            else:
                self.conf_path = eqlog_dir

    def read_xml(self, conf_path=''):
        """
        读取xml文件配置
        :param conf_path: 配置文件路径
        :return: None
        """
        conf_path = self.conf_path if conf_path == '' else conf_path
        now = datetime.datetime.now()
        try:  # 读取日志文件所在目录
            xml_tree = XMLTree.parse(conf_path)
            xml_root = xml_tree.getroot()
            self.logfile_path = xml_root.find('file_path').find(self.env).text
        except Exception as e:
            print(now.strftime("%Y-%m-%d %H:%M:%S"), '[WARNING]', '[eqlog]',
                  f"[{__file__}, {get_run_info().f_code.co_name}(), line {get_run_info().f_lineno}]",
                  '[Get log file path]', str(e))
        try:  # 读取控制台输出标志
            xml_tree = XMLTree.parse(conf_path)
            xml_root = xml_tree.getroot()
            self.console = True if xml_root.find('console').text == 'true' else False
        except Exception as e:
            print(now.strftime("%Y-%m-%d %H:%M:%S"), '[WARNING]', '[eqlog]',
                  f"[{__file__}, {get_run_info().f_code.co_name}(), line {get_run_info().f_lineno}]",
                  '[Get console output config]', str(e))



"""
日志工具类
"""

from logging import getLogger, FileHandler, Formatter, ERROR, DEBUG, INFO, WARN
from eqlog.common import FileOption, create_log_file
from functools import wraps
from time import strftime, localtime
import datetime


class Logger:
    level_dictionary = {  # 日志级别字典
        'INFO': INFO,
        'ERROR': ERROR,
        'DEBUG': DEBUG,
        'WARNING': WARN
    }

    def __init__(self, module_name='common', level='INFO', log_file_dir='', conf_file='', conf_deep=3,
                 conf_console=None):
        """
        日志工具初始化
        :param module_name: 日志模块名称
        :param level: 日志级别
        :param log_file_dir: 日志根目录
        :param conf_file: 配置文件
        """
        self.op = FileOption(conf_file, conf_deep)
        self.op.config_path_read()
        self.op.read_xml()
        if self.op.conf_path == '':  # 配置文件不存在
            print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), '[WARNING] 日志配置文件不存在!')
            self._root = log_file_dir if log_file_dir != '' else './logs/'
            self._console = conf_console if conf_console is not None else False
        else:
            self._root = self.op.logfile_path if self.op.logfile_path != '' else './logs/'  # 日志文件路径
            self._console = conf_console if conf_console is not None else self.op.console

        self._logger = getLogger()  # logger
        self._date_time = strftime("%Y-%m-%d", localtime())  # 时间后缀
        self._log_module_name = module_name  # 日志模块名
        file_name = self.set_log_file_name()  # 日志文件名
        create_log_file(file_name)  # 创建日志文件

        self._logger.setLevel(self.level_dictionary[level])  # 日志级别
        self._formatter = Formatter(
            "%(asctime)s [%(levelname)s] [%(filename)s %(funcName)s() line %(lineno)d] %(message)s")  # 日志格式
        self._file_handler = FileHandler(file_name, encoding='utf-8', mode='a')  # 日志文件编码
        self._file_handler.setFormatter(self._formatter)  # 日志文件格式

    # 函数装饰器：被装饰函数隐式输出日志
    # param func: 被装饰函数
    # return 装饰后的新函数
    def log(self, func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                self.common_log('info', '[Function ' + func.__name__ + '()] [Params]:' + (
                    'None' if (lambda x: not x)(args) else str(args)) + ', ' + (
                                    '' if (lambda x: not x)(kwargs) else str(kwargs)))
                return func(*args, **kwargs)
            except Exception as e:
                self.common_log('error', '[Function ' + func.__name__ + '()] [执行异常]:' + str(e))

        return wrapper

    def common_log(self, level='info', message=''):
        """
        通用日志处理
        :param level: 输出日志级别
        :param message: 日志字符串
        :return: None
        """
        # 日志时间
        if strftime("%Y-%m-%d", localtime()) == self._date_time:
            pass
        else:
            self._date_time = strftime("%Y-%m-%d", localtime())
            file_name = self.set_log_file_name()
            print('[eqlog] 当前日期已更新，重新生成配置文件:', file_name)
            self._file_handler = FileHandler(file_name, encoding='utf-8', mode='a')
            self._file_handler.setFormatter(self._formatter)
        self._logger.addHandler(self._file_handler)
        if self._console:
            print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), '[' + level.upper() + '] [eqlog]', message)
        # 日志级别
        if level == 'info':
            self._logger.info(message)
        elif level == 'warning':
            self._logger.warning(message)
        elif level == 'debug':
            self._logger.debug(message)
        else:
            self._logger.error(message)
        self._logger.removeHandler(self._file_handler)

    def set_log_file_name(self):
        """
        日志文件名设置
        :return: 日志文件完整路径
        """
        if self._log_module_name == 'common':
            file_name = self._root + 'common_' + self._date_time + '.log'
        else:
            file_name = self._root + self._log_module_name + '_' + self._date_time + '.log'
        return file_name

    def info(self, info):
        """
        info 级别日志
        :param info: 日志信息
        :return: None
        """
        self.common_log('info', info)

    def error(self, error):
        """
        error 级别日志
        :param error: 日志信息
        :return: None
        """
        self.common_log('error', error)

    def waring(self, waring):
        """
        waring 级别日志
        :param waring: 日志信息
        :return: None
        """
        self.common_log('warning', waring)

    def debug(self, debug):
        """
        debug 级别日志
        :param debug: 日志信息
        :return: None
        """
        self.common_log('debug', debug)



from setuptools import setup, find_packages

setup(
    name="eqlog",
    version="0.0.7",
    keywords=("pip", "eqlog", "log", "logger"),
    description="log tool",
    long_description="log tool",
    license="MIT Licence",

    url="https://gitee.com/jingenqiang/eqlog.git",
    author="eq",
    author_email="eq_enqiang@163.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[]
)

"""
项目打包
python setup.py bdist_egg     # 生成类似 eqlog-0.0.1-py2.7.egg，支持 easy_install 
# 使用此方式
python setup.py sdist         # 生成类似 eqlog-0.0.1.tar.gz，支持 pip
# twine 需要安装
twine upload dist/eqlog-0.0.6.tar.gz
"""
